using System.Collections;
using System.Collections.Generic;
using UnityEngine;

// GLGeometry (Dankook University Kyoung Shin Park 2023/09/21)
public class DrawGLGeometry : MonoBehaviour
{
    //private float speed = 2.0f;
    //private float rotSpeed = 2.0f;
    public Material mat;

    private void CreateMaterial()
	{
		if (!mat)
		{
			// Unity has a built-in shader that is useful for drawing
			// simple colored things.
			Shader shader = Shader.Find ("Hidden/Internal-Colored");
			mat = new Material (shader);
			mat.hideFlags = HideFlags.HideAndDontSave;
			// Turn on alpha blending
			mat.SetInt ("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.SrcAlpha);
			mat.SetInt ("_DstBlend", (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);
			// Turn backface culling
			mat.SetInt ("_Cull", (int)UnityEngine.Rendering.CullMode.Back);
			// Turn backface culling off
			//mat.SetInt ("_Cull", (int)UnityEngine.Rendering.CullMode.Off);
			// Turn off depth writes
			mat.SetInt ("_ZWrite", 0);
		}
	}

    private void Draw()
    {
        CreateMaterial();
        mat.SetPass(0);
        GLGeometry.Line(Color.yellow, new Vector3(3, 3, 0), new Vector3(-3, -3, 0));
        GLGeometry.Line(Color.yellow, new Vector3(3, 3, 0), new Vector3(-3, -3, 0));
        // CW
        GLGeometry.FilledTriangle(Color.red, Color.green, Color.blue, new Vector3(-2, 0, 0), new Vector3(-1, 2, 0), new Vector3(0, 0, 0));
        // CW
        GLGeometry.FilledTriangle(Color.red, Color.green, Color.blue, new Vector3(0, 0, 0), new Vector3(1, 2, 0), new Vector3(2, 0, 0));
        // CW
        GLGeometry.FilledTriangle(Color.red, Color.green, Color.blue, new Vector3(-2, -2, 0), new Vector3(-1, 0, 0), new Vector3(0, -2, 0));
        // CCW (back-face culling)
        GLGeometry.FilledTriangle(Color.red, Color.green, Color.blue, new Vector3(0, -2, 0), new Vector3(2, -2, 0), new Vector3(1, 0, 0)); // CCW - backface culling
        // CW
        GLGeometry.Rect(Color.red, Color.green, Color.blue, Color.yellow, new Vector3(2, 0, 0), new Vector3(2, 2, 0), new Vector3(4, 2, 0), new Vector3(4, 0, 0));
        // CCW (back-face culling)
        GLGeometry.Rect(Color.red, Color.green, Color.blue, Color.yellow, new Vector3(2, -2, 0), new Vector3(4, -2, 0), new Vector3(4, 0, 0), new Vector3(2, 0, 0));
        // not-coplanarv (still drawing)
        GLGeometry.Rect(Color.red, Color.green, Color.blue, Color.yellow, new Vector3(2, -4, 0), new Vector3(2, -2, 1), new Vector3(4, -2, 0), new Vector3(4, -4, 1));
        // Quad CW
        GLGeometry.Quad(Color.red, new Vector3(3, 4, 0), new Vector3(1, 0, -1), new Vector3(0, 1, 0), 2, 3);
        // CW
        GLGeometry.Circle(Color.cyan, new Vector3(-4, 0, 0), 2.5f, 16);
        // CW
        GLGeometry.Cylinder(Color.magenta, new Vector3(6, 0, 0), 1.5f, 4, 32);
        // CW
        GLGeometry.Sphere(Color.yellow, new Vector3(-4, 0, 0), 2, 32, 32);
        // CW
        GLGeometry.Torus(Color.white, 2, 0.5f, 16, 16);
        // CW
        GLGeometry.Cube(Color.blue, new Vector3(0, 4, 0), 1.5f);
        // CW
        GLGeometry.Parallelepiped(Color.green, new Vector3(-2, -4, 0), new Vector3(1, 0, 0), new Vector3(2, 2, 0), new Vector3(0, 0, 1));
    }

    // Start is called before the first frame update
    void Start()
    {
        
    }

    // Update is called once per frame
    void Update()
    {
        
    }

       // will be called after all regular rendering is done. (OpenGL)
    public void OnRenderObject()
    {
        Draw();
    }

    // Will be called from camera after regular rendering is done. (OpenGL)
    public void OnPostRender()
    {
        Draw();
    }

    // Draw Gizmos
    private void OnDrawGizmos()
    {
        Draw();
    }

    // Draw Gizmos when the game object is selected
    private void OnDrawGizmosSelected()
    {
        Draw();
    }
}
